@@ -125,6 +125,15 @@ lav_options_set <- function(opt = formals(lavaan)) {
         # nothing to do
     } else if(opt$missing == "available.cases") {
         # nothing to do, or warn if not categorical?
+    } else if(opt$missing == "doubly.robust") {
+        if(opt$estimator != "pml") {
+            stop("lavaan ERROR: doubly.robust option only available for estimator PML")
+        }
+    } else if(opt$missing == "doubly_robust") {
+        opt$missing <- "doubly.robust"
+        if(opt$estimator != "pml") {
+            stop("lavaan ERROR: doubly.robust option only available for estimator PML")
+        }
     } else if(opt$missing == "available_cases") {
         opt$missing <- "available.cases"
     } else {
@@ -587,7 +596,7 @@ lav_options_set <- function(opt = formals(lavaan)) {
             stop("lavaan ERROR: fixed.x = FALSE is not supported when conditional.x = TRUE.")
         }
     } else if(opt$fixed.x == "default") {
-        if(opt$estimator == "ML" && (opt$mimic == "Mplus" ||
+        if(opt$estimator %in% c("MML", "ML") && (opt$mimic == "Mplus" ||
                                      opt$mimic == "lavaan")) {
             opt$fixed.x <- TRUE
         } else if(opt$conditional.x) {
